{Sudoku-Solver
 Copyright (c) 2010 proteino.de, Norbert Schröder

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 2 of the license, or at your option any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRENTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 PARTICULAR PURPOSE. See the GNU General Public License for more details.
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
 675 Mass Ave, Cambridge, MA 02139, USA.}

unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, Menus, ExtCtrls, Spin, Unit2, EpikTimer;

type

  { TForm1 }

  TForm1 = class(TForm)
    btn_Autosolve: TButton;
    btn_Block: TButton;
    btn_clrDebuglist: TButton;
    btn_RowCol: TButton;
    btn_ShowArray: TButton;
    btn_ShowCand: TButton;
    btn_Twin: TButton;
    ChB_SaveProtection: TCheckBox;
    ChB_SolveList: TCheckBox;
    ChB_autoSave: TCheckBox;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    L_CntPuzzles: TLabel;
    ListBox1: TListBox;
    MainMenu1: TMainMenu;
    Memo1: TMemo;
    MenuItem1: TMenuItem;
    MenuDateiLaden: TMenuItem;
    MenuDateiSpeichern: TMenuItem;
    MenuItem2: TMenuItem;
    MenuHelp: TMenuItem;
    MenuAbout: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuDateiBeenden: TMenuItem;
    MenuExtrasNewSudoku: TMenuItem;
    MenuExtrasNewTetradoku: TMenuItem;
    MenuExtrasNewPentadoku: TMenuItem;
    OpenDialog1: TOpenDialog;
    Panel1: TPanel;
    Panel2: TPanel;
    RB_logic: TRadioButton;
    RB_backtrack: TRadioButton;
    SaveDialog1: TSaveDialog;
    SpEd_PuzzleNr: TSpinEdit;
    function  ClrArray(s:string):boolean;
    function  contradiction():boolean;
    procedure btn_ShowCandClick(Sender: TObject);
    procedure btn_ShowArrayClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject{;var CloseAction: TCloseAction});
    procedure btn_clrDebuglistClick(Sender: TObject);
    procedure ChB_SaveProtectionChange(Sender: TObject);
    procedure LoadSinglePuzzle(s:string;Sender: TObject);
    function  LoadPuzzleFromList(pNr:integer;Sender: TObject):integer;
    procedure MenuDateiLadenClick(Sender: TObject);
    procedure PuzzleGenerator(s:string;Sender: TObject);
    procedure MenuExtrasNewPentadokuClick(Sender: TObject);
    procedure MenuExtrasNewSudokuClick(Sender: TObject);
    procedure MenuExtrasNewTetradokuClick(Sender: TObject);
    procedure SpEd_PuzzleNrChange(Sender: TObject);
    procedure MenuDateiSpeichernClick(Sender: TObject);
    procedure MenuDateiBeendenClick(Sender: TObject);
    procedure SetValue(Sender: TObject;x,y,z:integer);
    procedure btn_RowColClick(Sender: TObject);
    procedure btn_TwinClick;
    procedure btn_BlockClick(Sender: TObject);
    procedure xDoku;
    procedure FastAutoSolve(Sender: TObject);
    procedure btn_AutosolveClick(Sender: TObject);
    procedure MenuAboutClick(Sender: TObject);
  private
    { private declarations }
  public
    ET1:TEpikTimer;
  end;

const maxVal=24;     // für max 25er Feldbreite
      version='v0.7';
type TFeld=record
 value:integer;      // if value known then value else "-1"
 cand:UInt64;        // binär gespeicherte Kandidaten, also max 64er Feldbreite
end;
var Form1:TForm1;
    PuzzleList,SolvedList:TStringList;        // AutoSolver with Save-Button checked puts the solved puzzles into SolvedList
    arr:array[0..maxVal,0..maxVal]of TFeld;   // the array for the doku
    arrB,arrX,arrY:array[0..maxVal]of UInt64; // Candidates for Block/Column/Row
    binv:array[0..maxVal]of UInt64;           // binary values as filter
    lv,bw:integer;  // lv-lastvalue;bw-blockwith
    fv:integer;     // fv-firstvalue, for sudoku-debug-prints, cause it not begins with 0
    kv:integer;     // known values
    tridoku,tetradoku,pentadoku:boolean;      // which doku is it?
    diagonale:boolean;     // X-Sudoku, available for all puzzle-types
    valuechanged:boolean;  // for autosolver: has the last run find a value?
    contradiction:boolean; // for autosolver: no value and no candidates -> problem
    filename:string; // storage for OpenDialog1.FileName so it can be used for saving SolvedList
implementation

function TForm1.ClrArray(s:string):boolean; // set: fv, lv, bw, arr[..].value:=-1; arr[..].cand:=all candidates
var b:boolean=false;
    x,y:integer;     // x,y=position, z=value
    u:UInt64;
begin
 if(s='sudoku')or(s='tridoku')then begin   // Standard 3x3 (Tridoku)
  tridoku:=true;tetradoku:=false;pentadoku:=false;
  fv:=1;lv:=8;bw:=3;kv:=81;b:=true;
  u:=511;
 end else if(s='tetradoku')or(s='hexadoku')then begin // 4x4, Hexadoku (nach Elektor)
  tridoku:=false;tetradoku:=true;pentadoku:=false;
  fv:=0;lv:=15;bw:=4;kv:=256;b:=true;
  u:=65535;
 end else if s='pentadoku'then begin       // 5x5
  tridoku:=false;tetradoku:=false;pentadoku:=true;
  fv:=0;lv:=24;bw:=5;kv:=625;b:=true;
  u:=33554431;
 end;
 if b then for x:=0to lv do begin
  arrB[x]:=u;arrX[x]:=u;arrY[x]:=u;
  for y:=0to lv do begin
   arr[x,y].value:=-1;
   arr[x,y].cand:=u;
   ClrArray:=true;
  end;
 end else ClrArray:=false;
end;

function TForm1.contradiction():boolean;
var i,j:integer;
begin
 contradiction:=false;
 for i:=0to lv do begin
  if arrX[i]>0then for j:=0to lv do if(arr[i,j].cand=0)and(arr[i,j].value=-1)then contradiction:=true;
  if arrY[i]>0then for j:=0to lv do if(arr[j,i].cand=0)and(arr[j,i].value=-1)then contradiction:=true;
 end;
end;

procedure TForm1.btn_ShowCandClick(Sender: TObject);
var x,y,z:integer;
    s:string;
begin
 ListBox1.Items.Add('-----------------------------------');
 ListBox1.Items.Add('Mögliche Kandidaten für die Felder:');
 for y:=0to lv do for x:=0to lv do begin
  s:='';
  for z:=0to lv do if(arr[x,y].cand OR binv[z])<>binv[z]then s+=inttostr(z+fv)+' ';
  if s<>''then ListBox1.Items.Add('('+inttostr(x)+','+inttostr(y)+'): '+s);
 end;
 ListBox1.Items.Add('-----------------------------------');
 ListBox1.Items.Add('Mögliche Kandidaten für die Spalten:');
 for x:=0to lv do begin
  s:='';
  for z:=0to lv do if(arrX[x] OR binv[z])<>binv[z]then s+=inttostr(z+fv)+' ';
  if s<>''then ListBox1.Items.Add('x='+inttostr(x)+': '+s);
 end;
 ListBox1.Items.Add('-----------------------------------');
 ListBox1.Items.Add('Mögliche Kandidaten für die Zeilen:');
 for y:=0to lv do begin
  s:='';
  for z:=0to lv do if(arrY[y] OR binv[z])<>binv[z]then s+=inttostr(z+fv)+' ';
  if s<>''then ListBox1.Items.Add('y='+inttostr(y)+': '+s);
 end;
 ListBox1.Items.Add('-----------------------------------');
 ListBox1.Items.Add('Mögliche Kandidaten für die Blöcke:');
 for y:=0to lv do begin
  s:='';
  for z:=0to lv do if(arrB[y] OR binv[z])<>binv[z]then s+=inttostr(z+fv)+' ';
  if s<>''then ListBox1.Items.Add('b='+inttostr(y)+': '+s);
 end;
 ListBox1.Items.Add('-----------------------------------');
 btn_ShowArrayClick(Sender);
end;

procedure TForm1.ChB_SaveProtectionChange(Sender: TObject);
begin
 if ChB_SaveProtection.Checked then Memo1.Enabled:=true else Memo1.Enabled:=false;
end;

procedure TForm1.btn_ShowArrayClick(Sender: TObject);
var h,x,y,z:integer;
    s:string;
begin
 Memo1.Clear;
 for y:=0to lv do begin
  s:='';
  h:=(y)mod bw;
  if(y>1)and(h=0)then begin
   if tridoku then s:='---x---x---'
   else if tetradoku then s:='----x----x----x----'
   else if pentadoku then s:='-----x-----x-----x-----x-----';
   Memo1.Lines.Add(s);
   s:='';
  end;
  for x:=0to lv do begin
   h:=x mod bw;
   if(x>1)and(h=0)then s+='|';
   z:=arr[x,y].value;
   if z<0then s+=' '
   else if tridoku then s+=char(z+49) // 49 cause normalized to 0-8
   else if tetradoku then begin
    if z<10then s+=char(z+48)else s+=char(z+87);
   end else if pentadoku then s+=char(z+97);
  end;
  Memo1.Lines.Add(uppercase(s));
 end;
end;

procedure TForm1.FormCreate(Sender: TObject); // binäre Filter basteln und ein paar Initialisierungen
var i:integer;
begin
 binv[0]:=-2; // binär alles auf 1 außer Bit 0, was 0 wird
 for i:=1to maxVal do binv[i]:=(binv[pred(i)]SHL 1)+1; // restliche Filter, eingeschobene 0 via +1 füllen
 ClrArray('sudoku');
 Form1.Caption:='Sudoku-Solver '+version;
 PuzzleList:=TStringList.Create;
 SolvedList:=TStringList.Create;
end;

procedure TForm1.FormClose(Sender: TObject{;var CloseAction: TCloseAction});
begin
 PuzzleList.Free;
 SolvedList.Free;
end;

procedure TForm1.btn_clrDebuglistClick(Sender: TObject);
begin
 ListBox1.Clear;
end;

procedure TForm1.LoadSinglePuzzle(s:string;Sender: TObject);
var x,y,z:integer;   // x,y=position, z=value
    a,b,i:integer; // Hilfsvariablen
begin
 y:=-1; // wird vor 1. Auswertung hochgesetzt
 i:=0;
 while(i<PuzzleList.Count-1)and(y<lv)do begin      // Zeilenweise auswerten, Gitter entfernen
  i+=1;
  s:=PuzzleList.Strings[i];
  if s[1]<>'-'then begin
   y+=1;
   s:=lowercase(s);
   x:=-1; // wird vor 1. Auswertung hochgesetzt
   for a:=1to length(s)do begin       // Zeichen auswerten
    z:=-1;
    b:=ord(s[a]);
    if(b>47)and(b<58)then begin       // Ziffern
     if(tridoku)and(b>48)then z:=b-49 // 49 cause normalized to 0-8
     else if tetradoku then z:=b-48;
    end;
    if(b>96)and(b<123)then begin     // Kleinbuchstaben
     if(tetradoku)and(b<103)then z:=b-87
     else if(pentadoku)and(b<122)then z:=b-97;
    end;
    if(z>-1)or(b=32)then begin       // beliebiger Wert oder Space=32
     x+=1;
//       if ChkBox_Verbose.Checked then ListBox1.Items.Add(inttostr(x)+','+inttostr(y)+':='+inttostr(z));
    end;
    if x>lv then begin
     ListBox1.Items.Add('Fehler in Datei.');
     exit;
    end else if z>=0then SetValue(Sender,x,y,z);
   end;
  end;
 end;
 SpEd_PuzzleNr.Value:=1;
end;

function TForm1.LoadPuzzleFromList(pNr:integer;Sender: TObject):integer; // pNr Puzzle-Number in List, Rückgabe ist Zeilennummer
var x,y,z,b,i,p1,p2:integer; // p1 = puzzle-counter; p2 = pos in der PuzzleList
    s:string;
begin
 p1:=0;p2:=0;
 while p1<pNr do begin
  if(PuzzleList.Strings[p1+p2][1]='[')then begin
   if(PuzzleList.Strings[p1+p2]='[Matrix=9x9]')then s:='sudoku'
   else if(PuzzleList.Strings[p1+p2]='[Matrix=16x16]')then s:='tetradoku'
   else if(PuzzleList.Strings[p1+p2]='[Matrix=25x25]')then s:='pentadoku'
   else if(PuzzleList.Strings[p1+p2]='[Diagonale=false]')then diagonale:=false
   else if(PuzzleList.Strings[p1+p2]='[Diagonale=true]')then diagonale:=true;
   p2+=1;
  end else p1+=1;
 end;
 p2+=p1-1;
 ClrArray(s);
 p1:=lv+1;p1*=p1;
 if(length(PuzzleList.Strings[p2])<>p1)then ListBox1.Items.Add('LoadPuzzleFromList: Falsche Vorgaben für Rätsel #'+inttostr(pNr))
 else begin
  s:='';
  x:=0;y:=0;
  for i:=1to p1 do begin
   if(PuzzleList.Strings[p2][i]<>'|')then begin
    b:=ord(PuzzleList.Strings[p2][i]);
    if(b>47)and(b<58)then begin       // Ziffern
     if(tridoku)and(b>48)then z:=b-49 // 49 cause normalized to 0-8
     else if tetradoku then z:=b-48;
    end;
    if(b>64)and(b<91)then begin     // Großbuchstaben
     if(tetradoku)and(b<71)then z:=b-55
     else if(pentadoku)and(b<90)then z:=b-65;
    end;
    if(b>96)and(b<123)then begin     // Kleinbuchstaben
     if(tetradoku)and(b<103)then z:=b-87
     else if(pentadoku)and(b<122)then z:=b-97;
    end;
    SetValue(Sender,x,y,z);
   end;
   x+=1;
   if x>lv then begin x:=0;y+=1;end;
  end;
  SpEd_PuzzleNr.Value:=pNr;
 end;
 LoadPuzzleFromList:=p2;
end;

procedure TForm1.MenuDateiLadenClick(Sender: TObject);
var s:string;
    a,b:integer;
begin
 if OpenDialog1.Execute then begin
  PuzzleList.LoadFromFile(OpenDialog1.FileName);
  filename:=OpenDialog1.FileName;
  s:=PuzzleList.Strings[0];
  if ClrArray(lowercase(s))then begin
   LoadSinglePuzzle(s,Sender);
   L_CntPuzzles.Caption:='1';
   SpEd_PuzzleNr.MaxValue:=1;
  end else if s[1]='['then begin
   SolvedList.LoadFromFile(OpenDialog1.FileName);
   LoadPuzzleFromList(1,Sender);
   b:=0;
   for a:=0to PuzzleList.Count-1do if PuzzleList.Strings[a][1]<>'[' then b+=1;
   L_CntPuzzles.Caption:=inttostr(b);
   SpEd_PuzzleNr.MaxValue:=b;
  end
  else ListBox1.Items.Add('Typ nicht erkannt, Rätsel nicht angelegt.');
 end;
 btn_ShowArrayClick(Sender);
end;

procedure TForm1.PuzzleGenerator(s:string;Sender: TObject);
var h_RB_logic,h_RB_backtrack:boolean;
    a,b,c,d,x,y,z:integer;
    arr_h:array[0..maxVal,0..maxVal]of integer;
    arr_h2:array[0..maxVal]of integer;
begin
 randomize;
 ListBox1.Items.Add('Hinweis: Das Erstellen von Sudokus kann auch auf schnellen Rechnern einige Sekunden oder in seltenen');
 ListBox1.Items.Add('Fällen sogar bis zu 1 Minute dauern. Das Erstellen von Tetradokus dauert in der Regel zwischen ca. 30s');
 ListBox1.Items.Add('und einigen Minuten. Bei Sudokus treten selten, bei Tetradokus häufig wesentlich längere Berechnungszeiten');
 ListBox1.Items.Add('auf. In diesen Fällen ist es empfehlenswert, das Programm per Taskmanager zu beenden. Aus');
 ListBox1.Items.Add('Performancegründen gibt es kaum Statusanzeigen. In seltenen Fällen tritt ein Zugriffsfehler auf, dessen');
 ListBox1.Items.Add('Ursache noch nicht geklärt ist. Programm dann bitte beenden und neu starten.');
 Application.ProcessMessages;
 h_RB_backtrack:=RB_backtrack.Checked;
 h_RB_logic:=RB_logic.Checked;
 RB_backtrack.Checked:=true;
 repeat
  ClrArray(s);
  for a:=0to lv*lv do SetValue(Sender,random(lv),random(lv),random(lv)); // random(lv) erstellt Werte von 0..lv
  btn_AutosolveClick(Sender);
  if kv>0then ListBox1.Items.Add('Rätsel nicht lösbar, neu versuchen');
  Application.ProcessMessages;
 until kv=0; // wir haben jetzt in arr ein gültiges Rätsel
 RB_backtrack.Checked:=h_RB_backtrack;
 RB_logic.Checked:=h_RB_logic;
 ListBox1.Items.Add('Vorlage erstellt');Application.ProcessMessages;

 c:=random(6);
 for a:=0to lv do for b:=0to lv do case c of // gespiegelt ins Hilfsarray
  0: arr_h[a,b]:=arr[a,b].value;
  1: arr_h[b,a]:=arr[a,b].value;
  2: arr_h[lv-a,b]:=arr[a,b].value;
  3: arr_h[a,lv-b]:=arr[a,b].value;
  4: arr_h[lv-b,a]:=arr[a,b].value;
  else arr_h[b,lv-a]:=arr[a,b].value; // else=5
 end;
 for d:=0to bw do begin
  a:=random(bw);b:=random(bw); // Spalten tauschen
  if a<>b then for c:=0to lv do begin
   arr_h2[c]:=arr_h[a+d*bw,c];
   arr_h[a+d*bw,c]:=arr_h[b+d*bw,c];
   arr_h[b+d*bw,c]:=arr_h2[c];
  end;
  a:=random(bw);b:=random(bw); // Zeilen tauschen
  if a<>b then for c:=0to lv do begin
   arr_h2[c]:=arr_h[c,a+d*bw];
   arr_h[c,a+d*bw]:=arr_h[c,b+d*bw];
   arr_h[c,b+d*bw]:=arr_h2[c];
  end;
 end;//ToDo: bei Bedarf noch Blockreihen/Blockspalten oder Ziffern tauschen
 for a:=0to 2*lv-1do arr_h[random(lv),random(lv)]:=-2; // einige Werte direkt entfernen
 ListBox1.Items.Add('Vorlage variiert');Application.ProcessMessages;

 d:=0;
 while d<10 do begin // per Zufallsgenerator Werte entfernen bis dies 10x fehlgeschalgen ist
  repeat
   x:=random(lv);y:=random(lv); // zufällige Koordinate bestimmen, Wert sichern, zum Test entfernen und das neue Rätsel testen
   z:=arr_h[x,y];
  until z>=0;
  arr_h[x,y]:=-2;
  ClrArray(s);
  for b:=0to lv do for a:=0to lv do if arr_h[a,b]>=0then SetValue(Sender,a,b,arr_h[a,b]);
  FastAutoSolve(Sender);
  if kv>0then begin // entfernter Wert muss wieder rein, da der Logiklöser das Rätsel nicht mehr hinbekommt
   d+=1;
   arr_h[x,y]:=z;
  end;
 end;
 ListBox1.Items.Add('Vorlage zufällig reduziert');Application.ProcessMessages;

 for y:=0to lv do for x:=0to lv do if arr_h[x,y]>=0then begin // systematisch alle übrigen Werte probieren
  z:=arr_h[x,y];
  arr_h[x,y]:=-2;
  ClrArray(s);
  for b:=0to lv do for a:=0to lv do if arr_h[a,b]>=0then SetValue(Sender,a,b,arr_h[a,b]);
  FastAutoSolve(Sender);
  if kv>0then arr_h[x,y]:=z; // entfernter Wert muss wieder rein, da der Logiklöser das Rätsel nicht mehr hinbekommt
 end;
 ListBox1.Items.Add('Rätsel erstellt');Application.ProcessMessages;

 ClrArray(s);
 for b:=0to lv do for a:=0to lv do if arr_h[a,b]>=0then SetValue(Sender,a,b,arr_h[a,b]); // fertig reduziertes Rätsel
 btn_ShowArrayClick(Sender);
end;

procedure TForm1.MenuExtrasNewSudokuClick(Sender: TObject);
begin
 PuzzleGenerator('sudoku',Sender);
end;

procedure TForm1.MenuExtrasNewTetradokuClick(Sender: TObject);
begin
 PuzzleGenerator('tetradoku',Sender);
end;

procedure TForm1.MenuExtrasNewPentadokuClick(Sender: TObject);
begin
 PuzzleGenerator('pentadoku',Sender);
end;

procedure TForm1.SpEd_PuzzleNrChange(Sender: TObject);
begin
 LoadPuzzleFromList(SpEd_PuzzleNr.Value,Sender);
 btn_ShowArrayClick(Sender);
end;

procedure TForm1.MenuDateiSpeichernClick(Sender: TObject);
var f:textfile;
    a,i:integer;
    s:string;
begin
 if SaveDialog1.Execute then begin
  assignfile(f,SaveDialog1.FileName);
  rewrite(f);
  if tridoku then begin writeln(f,'Sudoku');a:=10 end
  else if tetradoku then begin writeln(f,'Tetradoku');a:=18 end
  else if pentadoku then begin writeln(f,'Pentadoku');a:=28 end;
  for i:=0to a do begin
   s:=Memo1.Lines[i];
   writeln(f,s);
  end;
  closefile(f);
 end;
end;

procedure TForm1.MenuDateiBeendenClick(Sender: TObject);
begin
  close;
end;

procedure TForm1.SetValue(Sender: TObject;x,y,z:integer); // set the value and remove the availability from row/column/block
var a,b,a1,b1:integer;
begin
 if((arr[x,y].cand OR binv[z])<>binv[z])and(arr[x,y].value<0)then begin
  valuechanged:=true;
  arr[x,y].value:=z;
  arr[x,y].cand:=0;
  kv-=1;
  // Verzicht auf Schleifen bringt an dieser Stelle deutlich mehr Performance:
  // 3x3-Sudoku 1-2% bei Zeilen+Spalten und noch mal ca. 4,4% bei Blöcken; insg. ca. 6,5%
  // 4x4-Tetradoku wird insg. ca. 4.8% schneller
  arr[x,0].cand:=arr[x,0].cand AND binv[z]; // column
  arr[x,1].cand:=arr[x,1].cand AND binv[z];
  arr[x,2].cand:=arr[x,2].cand AND binv[z];
  arr[x,3].cand:=arr[x,3].cand AND binv[z];
  arr[x,4].cand:=arr[x,4].cand AND binv[z];
  arr[x,5].cand:=arr[x,5].cand AND binv[z];
  arr[x,6].cand:=arr[x,6].cand AND binv[z];
  arr[x,7].cand:=arr[x,7].cand AND binv[z];
  arr[x,8].cand:=arr[x,8].cand AND binv[z];
  arr[0,y].cand:=arr[0,y].cand AND binv[z]; // row
  arr[1,y].cand:=arr[1,y].cand AND binv[z];
  arr[2,y].cand:=arr[2,y].cand AND binv[z];
  arr[3,y].cand:=arr[3,y].cand AND binv[z];
  arr[4,y].cand:=arr[4,y].cand AND binv[z];
  arr[5,y].cand:=arr[5,y].cand AND binv[z];
  arr[6,y].cand:=arr[6,y].cand AND binv[z];
  arr[7,y].cand:=arr[7,y].cand AND binv[z];
  arr[8,y].cand:=arr[8,y].cand AND binv[z];
  if bw>3then for a:=9to lv do begin
   arr[x,a].cand:=arr[x,a].cand AND binv[z]; // column
   arr[a,y].cand:=arr[a,y].cand AND binv[z]; // row
  end;
  b:=(y div bw)*bw; // block: get position of upper left corner
  a:=x div bw;      // ohne bw, da a noch für arrB benötigt wird
  arrX[x]:=arrX[x] AND binv[z];
  arrY[y]:=arrY[y] AND binv[z];
  arrB[b+a]:=arrB[b+a] AND binv[z];
  a*=bw;

  arr[a,  b  ].cand:= arr[a,  b  ].cand AND binv[z];
  arr[a,  b+1].cand:= arr[a,  b+1].cand AND binv[z];
  arr[a,  b+2].cand:= arr[a,  b+2].cand AND binv[z];
  arr[a+1,b  ].cand:= arr[a+1,b  ].cand AND binv[z];
  arr[a+1,b+1].cand:= arr[a+1,b+1].cand AND binv[z];
  arr[a+1,b+2].cand:= arr[a+1,b+2].cand AND binv[z];
  arr[a+2,b  ].cand:= arr[a+2,b  ].cand AND binv[z];
  arr[a+2,b+1].cand:= arr[a+2,b+1].cand AND binv[z];
  arr[a+2,b+2].cand:= arr[a+2,b+2].cand AND binv[z];
  if bw>3then begin // tetradoku wird so ca. 3% schneller
   arr[a,  b+3].cand:=arr[a,  b+3].cand AND binv[z];
   arr[a+1,b+3].cand:=arr[a+1,b+3].cand AND binv[z];
   arr[a+2,b+3].cand:=arr[a+2,b+3].cand AND binv[z];
   arr[a+3,b  ].cand:=arr[a+3,b  ].cand AND binv[z];
   arr[a+3,b+1].cand:=arr[a+3,b+1].cand AND binv[z];
   arr[a+3,b+2].cand:=arr[a+3,b+2].cand AND binv[z];
   arr[a+3,b+3].cand:=arr[a+3,b+3].cand AND binv[z];
   if bw>4then begin
    arr[a,  b+4].cand:=arr[a,  b+4].cand AND binv[z];
    arr[a+1,b+4].cand:=arr[a+1,b+4].cand AND binv[z];
    arr[a+2,b+4].cand:=arr[a+2,b+4].cand AND binv[z];
    arr[a+3,b+4].cand:=arr[a+3,b+4].cand AND binv[z];
    arr[a+4,b  ].cand:=arr[a+4,b  ].cand AND binv[z];
    arr[a+4,b+1].cand:=arr[a+4,b+1].cand AND binv[z];
    arr[a+4,b+2].cand:=arr[a+4,b+2].cand AND binv[z];
    arr[a+4,b+3].cand:=arr[a+4,b+3].cand AND binv[z];
    arr[a+4,b+4].cand:=arr[a+4,b+4].cand AND binv[z];
    if bw>5then for b1:=b+5to pred(b+bw)do for a1:=a+5to pred(a+bw)do
     if(a1<>x)or(b1<>y)then arr[a1,b1].cand:=arr[a1,b1].cand AND binv[z];
   end
  end
 end //else ListBox1.Items.Add('SetValue, Error: Can''t set ('+inttostr(x)+','+inttostr(y)+'):='+inttostr(z+fv));
end;

procedure TForm1.btn_RowColClick(Sender: TObject); // Strategien: Hidden Single, Naked Single (nur für Zeilen/Spalten)
var a,b,i,lastxy:integer;
    z:integer; // Counter für Kandidaten in x/y (bei 1 Hidden Single, wird auch von btn_BlockClick gefunden)
begin
 for b:=0to lv do begin
  if arrY[b]>0then begin // Zeilen checken
   for i:=0to lv do if(arrY[b] OR binv[i])<>binv[i]then begin // nur übrige Kandidaten
    z:=0;
    for a:=0to lv do if(arr[a,b].cand OR binv[i])<>binv[i]then begin // gucken, ob der Kandidat i im Feld a,b noch übrig ist
     if arr[a,b].cand=(NOT binv[i])then begin // Naked Single
      SetValue(Sender,a,b,i);
      z:=0; // ist notwendig, falls der Kandidat vor dem Naked Single schon 1x in einem anderen Feld gefunden wurde -> sonst false positive Hidden Single
      break;
     end else begin
      z+=1;lastxy:=a;
     end;
    end;
    if z=1then SetValue(Sender,lastxy,b,i); // Hidden Single
   end;
  end;
  if arrX[b]>0then begin // Spalten checken
   for i:=0to lv do if(arrX[b] OR binv[i])<>binv[i]then begin
    z:=0;
    for a:=0to lv do if(arr[b,a].cand OR binv[i])<>binv[i]then begin
     if arr[b,a].cand=(NOT binv[i])then begin // Naked Single
      SetValue(Sender,b,a,i);
      z:=0;
      break;
     end else begin
      z+=1;lastxy:=a;
     end;
    end;
    if z=1then SetValue(Sender,b,lastxy,i); // Hidden Single
   end;
  end;
 end;
end;

procedure TForm1.btn_TwinClick; // Strategien: Naked Pair, Naked Triple (letztere unvollständig)
type Tkoords=record
 x,y:integer; // Koordinaten
 z:uint64;    // Kandidaten
 a:integer;   // Anzahl der Kandidaten -> vereinfacht die Verarbeitung, da man nicht mehrfach zählen muss
end;
var a,b,a1,b1,a2,b2,z,k:integer;
    anz:integer; // Anzahl interessanter Felder, die in arrh abgelegt werden
    arrh:array[0..maxVal]of Tkoords; // Hilfs-Array für Twinsearch
    twinfound:boolean;

 procedure HilfsarrayVerwurster;inline; // eignet sich für Hilfsarray-Wurst in Blöcken, Zeilen und Spalten
 var a3,b3,c3,i:integer;
 begin
  if anz>2then begin // untersuchen, ob identische Kandidatengruppen vorhanden sind und entsprechende Kandidaten aus den restlichen Feldern entfernen
   for a3:=0to anz-2do for b3:=succ(a3)to anz-1do // jeden mit jedem anderen(!) vergleichen; da anz nicht mit 0 beginnt, dürfen die Schleifen nur bis anz-1 zählen
    if(arrh[a3].z XOR arrh[b3].z)=0then begin // 2 Felder mit identischen Kandidaten :)
     if arrh[a3].a=2then begin                // simpler Twin: nur 2 Kandidaten
      for i:=0to anz-1do if(i<>a3)and(i<>b3)then begin // die 2 Kandidaten aus den Kandidatenlisten der anderen noch offenen Felder entfernen
       arrh[i].z:=(NOT arrh[a3].z)AND arrh[i].z; // evtl. müsste die Kandidatenanzahl in den geänderten Feldern noch angepasst werden?
       twinfound:=true;
      end;
     end
     else if arrh[a3].a=3then begin           // Twin mit 3 Kandidaten, hier einen Kopfstand machen und dann:
      for c3:=0to anz-1do if(c3<>a3)and(c3<>b3)then begin // in arrh ein Feld suchen, welches entweder identisch ist oder gar einen Kandidaten weniger besitzt
       if(arrh[c3].z XOR arrh[a3].z)=0then begin // Test, ob 3. Feld mit gleichen Kandidaten existiert
        for i:=0to anz-1do if(i<>a3)and(i<>b3)and(i<>c3)then begin // die 3 Kandidaten aus den Kandidatenlisten der anderen noch offenen Felder entfernen
         arrh[i].z:=(NOT arrh[a3].z)AND arrh[i].z; // evtl. müsste die Kandidatenanzahl in den geänderten Feldern noch angepasst werden?
         twinfound:=true;
        end;
       end;
       //!!! Test, ob 3. Feld mit genau 2 der 3 Kandidaten existiert - das ist ekelig und fehlt vorerst
      end;
     end;
   end;
   if twinfound then for i:=0to anz-1do begin
    arr[arrh[i].x,arrh[i].y].cand:=arrh[i].z; // Zurückschreiben der Änderungen ins große Array
   end;
  end;
 end;

begin
// Blockschleife -> Felder mit 2, 3 oder mehr Kandidaten suchen
 for b1:=0to pred(bw)do for a1:=0to pred(bw)do begin // block-(corner-)loops
  a:=a1*bw;b:=b1*bw;      // Blockecken in (a,b)
  anz:=0;                 // Größe des Hilfsarrays vorm Block 0en
  twinfound:=false;
  for b2:=b to pred(b+bw)do for a2:=a to pred(a+bw)do begin // jetzt sind wir in den Blöcken
   k:=0;
   for z:=0to lv do if(arr[a2,b2].cand OR binv[z])<>binv[z]then k+=1;
   if k>1then begin       // k=0 -> Block gelöst, k=1 -> Fehler/nicht lösbar
    arrh[anz].x:=a2;
    arrh[anz].y:=b2;
    arrh[anz].z:=arr[a2,b2].cand;
    arrh[anz].a:=k;
    anz+=1;
   end;
  end;
  HilfsarrayVerwurster;
 end;

// Zeilen/Spalten -> Felder mit 2, 3 oder mehr Kandidaten suchen
 for b1:=0to lv do begin  // äußere Schleife
  anz:=0;                 // Größe des Hilfsarrays 0en
  twinfound:=false;
  for a1:=0to lv do begin // Zeilen -> innere Schleife
   k:=0;
   for z:=0to lv do if(arr[a1,b1].cand OR binv[z])<>binv[z]then k+=1;
   if k>1then begin       // k=0 -> Zeilen gelöst, k=1 -> Fehler/nicht lösbar
    arrh[anz].x:=a1;
    arrh[anz].y:=b1;
    arrh[anz].z:=arr[a1,b1].cand;
    arrh[anz].a:=k;
    anz+=1;
   end;
  end;
  HilfsarrayVerwurster;
  anz:=0;                 // Größe des Hilfsarrays 0en
  twinfound:=false;
  for a1:=0to lv do begin // Spalten -> innere Schleife
   k:=0;
   for z:=0to lv do if(arr[b1,a1].cand OR binv[z])<>binv[z]then k+=1;
   if k>1then begin       // k=0 -> Spalte gelöst, k=1 -> Fehler/nicht lösbar
    arrh[anz].x:=b1;
    arrh[anz].y:=a1;
    arrh[anz].z:=arr[b1,a1].cand;
    arrh[anz].a:=k;
    anz+=1;
   end;
  end;
  HilfsarrayVerwurster;
 end;
end;

procedure TForm1.btn_BlockClick(Sender: TObject); // Strategien: Block-Line-Interaction, Hidden Single, Naked Single (die letzten beiden nur für Blöcke)
{- Kandidaten ermitteln, die in einem Block nur noch in einer Zeile/Spalte auftreten können
 - zu den betreffenden Kandidaten die entsprechende Spalte/Zeile ermitteln
   -> in den anderen Blöcken diesen Kandidaten in der jeweiligen Zeile/Spalte sperren
   -> Strategie Block-Line-Interaction
 - dadurch indirekt Strategie Hidden Single integriert
 - nebenbei Test auf Naked Single}
var a,b,a1,b1,a2,b2,a3,b3,i,lastx,lasty:integer;
    c:uint64;//Kandidaten im Block
    arrXB,arrYB:array[0..7]of uint64;// Kandidaten in X/Y des Blocks, max. 8 Zeilen/Spalten bei uint64
    xb,yb:integer; // speichert, in wie vielen x- bzw. y-Werten ein Kandidat enthalten ist
begin
 for b1:=0to pred(bw)do for a1:=0to pred(bw)do begin    // block-(corner-)loops
  a:=a1*bw;b:=b1*bw;      // Blockecken in (a,b)
  c:=arrB[b+a1];          // mgl. Kandidaten im aktuellen Block                 //if ChkBox_Verbose.Checked then ListBox1.Items.Add('PairClick, Info: Block '+inttostr(b+a1));
  for i:=0to pred(bw)do begin arrXB[i]:=0;arrYB[i]:=0;end;  // Init der Arrays
  for i:=0to lv do if(c OR binv[i])<>binv[i]then begin // einzeln über die vorhandenen Kandidaten des Blocks gehen
   for b2:=b to pred(b+bw)do if(arrY[b2] OR binv[i])<>binv[i]then for a2:=a to pred(a+bw)do begin //if ChkBox_Verbose.Checked then ListBox1.Items.Add('PairClick, Info: untersuchte Koords '+inttostr(a2)+','+inttostr(b2));
    if(arr[a2,b2].cand OR binv[i])<>binv[i]then begin
     if arr[a2,b2].cand=(NOT binv[i])then begin // Strategie "Naked Single"
      SetValue(Sender,a2,b2,i);                                                 //if ChkBox_Verbose.Checked then ListBox1.Items.Add('PairClick, Info: Block '+inttostr(b+a1)+'; ('+inttostr(a2)+','+inttostr(b2)+'):='+inttostr(i+fv)+' "Naked Single"');
      for a3:=0 to pred(bw)do begin             // i aus arrXB und arrYB entfernen
       arrXB[a3]:=arrXB[a3] OR NOT binv[i];
       arrYB[a3]:=arrYB[a3] OR NOT binv[i];
      end;
     end else begin
      a3:=a2-a;b3:=b2-b;
      arrXB[a3]:=arrXB[a3] OR NOT binv[i]; // mgl. Kandidaten der Spalte(n)
      arrYB[b3]:=arrYB[b3] OR NOT binv[i]; // mgl. Kandidaten der Zeile(n)
     end;
    end;
   end; // jetzt sind alle Kandidaten des Blocks auch nach ihren mgl. Zeilen/Spalten abrufbar
   xb:=0;yb:=0;
   for a2:=0 to pred(bw)do begin
    if(arrXB[a2] OR binv[i])<>binv[i]then begin
     xb+=1;       // in wie vielen Spalten gibt es den Kandidaten i?
     lastx:=a2+a; // falls nach Schleifenende xb=1, dann haben wir die Spalte auf lastx
    end;
    if(arrYB[a2] OR binv[i])<>binv[i]then begin
     yb+=1;       // in wie vielen Zeilen gibt es den Kandidaten i?
     lasty:=a2+b; // falls nach Schleifenende yb=1, dann haben wir die Zeile auf lasty
    end;
   end;
   if xb=1then begin          // Kandidat i kann nur noch in einer Spalte stehen //if ChkBox_Verbose.Checked then ListBox1.Items.Add('PairClick, Info: Block '+inttostr(b+a1)+', '+inttostr(i+fv)+' kann nur noch in einer Spalte stehen');
    if yb=1then begin         // Kandidat i kann nur noch in genau einer Zeile UND genau einer Spalte stehen => Strategie "Hidden Single" (auch Teil von btn_BlockClick), inkludiert Strategien "Full House" und "Last Digit"
     SetValue(Sender,lastx,lasty,i);                                            //if ChkBox_Verbose.Checked then ListBox1.Items.Add('PairClick, Info: Block '+inttostr(b+a1)+'; ('+inttostr(lastx)+','+inttostr(lasty)+'):='+inttostr(i+fv)+' "Hidden Single"');
    end else begin            // ein Kandidat kann nur noch in genau einer Spalte, aber mehreren Zeilen stehen => Kandidat kann nicht in gleicher (globaler) Spalte der anderen Blöcke stehen
     for b2:=0to lv do if(b2<b)or(b2>pred(b+bw))then begin // in den anderen Blöcken sperren //if ChkBox_Verbose.Checked then ListBox1.Items.Add('PairClick, Info: Block '+inttostr(b+a1)+', '+inttostr(i+fv)+' wird auf '+inttostr(a2)+','+inttostr(b2)+' gesperrt');
      arr[lastx,b2].cand:=arr[lastx,b2].cand AND binv[i];
     end;
    end
   end else if yb=1then begin // ein Kandidat kann nur noch in genau einer Zeile, aber mehreren Spalten stehen => Kandidat kann nicht in gleicher (globaler) Zeile der anderen Blöcke stehen //if ChkBox_Verbose.Checked then ListBox1.Items.Add('PairClick, Info: Block '+inttostr(b+a1)+', '+inttostr(i+fv)+' kann nur noch in einer Zeile stehen');
    for a2:=0to lv do if(a2<a)or(a2>pred(a+bw))then begin // in den anderen Blöcken sperren //if ChkBox_Verbose.Checked then ListBox1.Items.Add('PairClick, Info: Block '+inttostr(b+a1)+', '+inttostr(i+fv)+' wird auf '+inttostr(a2)+','+inttostr(b2)+' gesperrt');
     arr[a2,lasty].cand:=arr[a2,lasty].cand AND binv[i];
    end;
   end;
  end;
  // hier könnten weitere Strategien hin, die arrXB/arrYB nutzen könnten
 end;
end;

procedure TForm1.xDoku;
var a,b:integer;
    c1,c2:uint64;       //Kandidaten in den Diagonalen
begin
 c1:=0;c2:=0;
 for a:=0to lv do begin // vorhandene Werte sammeln
  b:=arr[a,a].value;
  if b>=0then c1:=c1 OR NOT binv[b];
  b:=arr[a,lv-a].value;
  if b>=0then c2:=c2 OR NOT binv[b];
 end;
 c1:=not c1;
 c2:=not c2;            // Invertieren der vorhandenen Werte ergibt noch offene Kandidaten
 for a:=0to lv do begin // ToDo: testen, ob Suche nach Naked/Hidden Singles hier einen Performanceschub bringt
  if arr[a,a].value<0then arr[a,a].cand:=arr[a,a].cand AND c1;
  if arr[a,lv-a].value<0then arr[a,lv-a].cand:=arr[a,lv-a].cand AND c2; // reduzierte Kandidatenliste
 end;
end;

procedure TForm1.FastAutoSolve(Sender: TObject);
var maxr:integer;
begin
 maxr:=0;
 repeat
  valuechanged:=false;
  maxr+=1;
  if kv>0then begin
   btn_BlockClick(Sender);
   btn_RowColClick(Sender);
   btn_BlockClick(Sender);
   if(not valuechanged)and(kv>0)then begin
    if diagonale then begin
     xDoku;
     btn_BlockClick(Sender);
    end;
    if(not valuechanged)then begin
     btn_TwinClick; // zeitaufwendige Funktion; nur aufrufen, wenn unbedingt nötig
     btn_RowColClick(Sender);
     btn_BlockClick(Sender);
     if(not valuechanged)then if(not RB_backtrack.Checked)then if contradiction then ListBox1.Items.Add('FastAutoSolve, Error: Puzzle #'+inttostr(SpEd_PuzzleNr.Value)+' is not a true sudoku-puzzle');
    end;
   end;
  end;
 until(maxr>20)or not valuechanged;
end;

procedure TForm1.btn_AutosolveClick(Sender: TObject);
var t1,t2:extended;
    a,x,y,z,zeile:integer;
    s,s2:string;

 procedure BackTrack;
 var bt_arr:array[0..maxVal,0..maxVal]of TFeld;   // the array for the doku
     bt_arrB,bt_arrX,bt_arrY:array[0..maxVal]of UInt64; // Candidates for Block/Column/Row
     bt_kv:integer;
     a,b,i,r:integer;
 begin
  bt_arr:=arr;bt_arrB:=arrB;bt_arrX:=arrX;bt_arrY:=arrY;bt_kv:=kv; // Backup for Backtracking
  for r:=0to 1do begin // r=0 ohne Rekursion, r=1 mit
   a:=0;b:=0;
   while(a<=lv)and(b<=lv)do begin // wir gehen über's Array
    if arr[a,b].cand>0then begin // und suchen eine Stelle mit offenen Kandidaten
     for i:=0to lv do if(arr[a,b].cand OR binv[i])<>binv[i]then begin
      SetValue(Sender,a,b,i);
      FastAutoSolve(Sender);
      if contradiction then begin
       arr:=bt_arr;arrB:=bt_arrB;arrX:=bt_arrX;arrY:=bt_arrY;kv:=bt_kv; // Restore the Backup
       break;
      end;
      if kv>0then begin // der Kandidat war (bisher) nicht so der Bringer
       if r=1then BackTrack; // yiha, wir rufen uns solange selbst auf, bis wir was cooles gefunden haben ^^
       if kv>0then begin // wenn nach der Rekursion immer noch kv>0 gilt, dann war der Kandidat Mist
//       Application.ProcessMessages;// damit man das Fenster noch bewegen kann ;)
//       btn_ShowArrayClick(Sender);
        arr:=bt_arr;arrB:=bt_arrB;arrX:=bt_arrX;arrY:=bt_arrY;kv:=bt_kv; // Restore the Backup
       end;
      end;
      if kv=0then begin // wenn man wissen will, ob es noch mehr Lösungen gibt, muss statt dem if-Kram ein else gesetzt werden. Nur an der Anzeige hapert es dann noch ;)
       a:=lv+1;b:=a; // tata - fertisch :)
       break;
      end;
     end;
    end;
    a+=1;if a=lv+1then begin a:=0;b+=1;end; // der Ersatz für den Automatismus einer verschachtelten for-Schleife
   end; // while
   if kv=0then break; // falls bei r=0 was gefunden wurde, muss man ja nicht unbedingt noch mal rekursiv suchen ;)
  end;
  if kv>0then begin // alle noch offenen Stellen mit allen noch offenen Kandidaten probiert und keine Lösung gefunden :(
   arr:=bt_arr;arrB:=bt_arrB;arrX:=bt_arrX;arrY:=bt_arrY;kv:=bt_kv; // Restore the Backup
  end;
 end;

begin
 if not(ChB_SolveList.Checked)then begin
  s:='Puzzle #'+inttostr(SpEd_PuzzleNr.Value);
  ET1:=TEpikTimer.create(nil);
  ET1.Clear;ET1.Start;
  t1:=ET1.Elapsed;
  FastAutoSolve(Sender);
  if(kv>0)then if(RB_backtrack.Checked)then BackTrack;
  t2:=ET1.Elapsed;
  ListBox1.Items.Add(s+', Rechenzeit: '+format('%.0n',[extended(t2-t1)*1000000])+' µs');
  btn_ShowArrayClick(Sender);
 end else begin // komplette Liste lösen
  ET1:=TEpikTimer.create(nil);
  ET1.Clear;ET1.Start;
  t1:=ET1.Elapsed;
  for a:=1to SpEd_PuzzleNr.MaxValue do begin
   zeile:=LoadPuzzleFromList(a,Sender);
   FastAutoSolve(Sender);
   if(kv>0)then if(RB_backtrack.Checked)then BackTrack;
   if ChB_autoSave.Checked then begin // Lösungen in extra StringList ablegen
    s2:='';
    for y:=0to lv do for x:=0to lv do begin
     z:=arr[x,y].value;
     if z<0then s2+='|'
     else if tridoku then s2+=char(z+49) // 49 cause normalized to 0-8
     else if tetradoku then begin
      if z<10then s2+=char(z+48)else s2+=char(z+87);
     end else if pentadoku then s2+=char(z+97);
    end;
    SolvedList[zeile]:=uppercase(s2);
   end;
  end;
  t2:=ET1.Elapsed;
  SolvedList.SaveToFile(filename+'.lsg');
  ListBox1.Items.Add(s+'Rechenzeit: '+format('%.0n',[extended(t2-t1)*1000000])+' µs');
  btn_ShowArrayClick(Sender);
 end;
end;

procedure TForm1.MenuAboutClick(Sender: TObject);
begin
 Form2.Show;
 Form2.Caption:='Sudoku-Solver '+version+' About';
end;

initialization
  {$I unit1.lrs}

end.

